/* FILE: util.cc                 -*-Mode: c++-*-
 *
 * Small utility classes
 *
 */

#include <string.h>

#include "nb.h"
#include "oc.h"
#include "threevector.h"
#include "util.h"

/* End includes */

////////////////////////////////////////////////////////////////////////
// Oxs_SplitList
Oxs_SplitList::Oxs_SplitList()
{
  argc=0;
  argv=NULL;
}

Oxs_SplitList::~Oxs_SplitList()
{
  if(argv!=NULL) Tcl_Free((char *)argv);
}

int Oxs_SplitList::Split(const char* argstr)
{
  if(argv!=NULL) {
    Tcl_Free((char *)argv);
    argv=NULL;
  }
  argc=0;
  char *str = new char[strlen(argstr)+1];
  strcpy(str,argstr);
  int errcode=Tcl_SplitList(NULL,str,&argc,&argv);
  delete[] str;
  return errcode;
}

const char* Oxs_SplitList::operator[](int n) const
{
  if(n<0 || n>=argc || argv==NULL) return NULL;
  return argv[n];
}

////////////////////////////////////////////////////////////////////////
// Oxs_FilePointer
Oxs_FilePointer::Oxs_FilePointer
(const char* filename,const char* filemode)
{
  fptr = fopen(filename,filemode);
  // Throw exception here if fptr == NULL ?
}

Oxs_FilePointer::~Oxs_FilePointer()
{
  if(fptr!=NULL) fclose(fptr);
}

////////////////////////////////////////////////////////////////////////
// Oxs_Box
void Oxs_Box::Set(REAL8m x0p,REAL8m x1p,
		  REAL8m y0p,REAL8m y1p,
		  REAL8m z0p,REAL8m z1p)
{
  if(x0p<=x1p) { x0=x0p; x1=x1p; }
  else         { x0=x1p; x1=x0p; }
  if(y0p<=y1p) { y0=y0p; y1=y1p; }
  else         { y0=y1p; y1=y0p; }
  if(z0p<=z1p) { z0=z0p; z1=z1p; }
  else         { z0=z1p; z1=z0p; }
}

void Oxs_Box::Set(const string& sx0,const string& sx1,
		  const string& sy0,const string& sy1,
		  const string& sz0,const string& sz1)
{
  REAL8m x0p,x1p,y0p,y1p,z0p,z1p;
  x0p=Nb_Atof(sx0.c_str());
  x1p=Nb_Atof(sx1.c_str());
  y0p=Nb_Atof(sy0.c_str());
  y1p=Nb_Atof(sy1.c_str());
  z0p=Nb_Atof(sz0.c_str());
  z1p=Nb_Atof(sz1.c_str());
  Set(x0p,x1p,y0p,y1p,z0p,z1p);
}

INT4m Oxs_Box::Expand(const Oxs_Box& other)
{ // Expands *this as necessary to contain region specified by "other".
  // Returns 1 if *this is enlarged, 0 if other was already contained in
  // *this.
  INT4m expanded=0;
  if(other.x0>other.x1) return 0; // "other" is an empty box
  if(x0>x1) {
    // *this is an empty box
    *this = other;
    return 1;
  }
  if(other.x0<x0) { x0 = other.x0; expanded=1; }
  if(other.y0<y0) { y0 = other.y0; expanded=1; }
  if(other.z0<z0) { z0 = other.z0; expanded=1; }
  if(other.x1>x1) { x1 = other.x1; expanded=1; }
  if(other.y1>y1) { y1 = other.y1; expanded=1; }
  if(other.z1>z1) { z1 = other.z1; expanded=1; }
  return expanded;
}

BOOL Oxs_Box::IsIn(const ThreeVector& point)
{
  if(point.x<x0 || point.x>x1 ||
     point.y<y0 || point.y>y1 ||
     point.z<z0 || point.z>z1) return 0;
  return 1;
}

BOOL Oxs_Box::IsContained(const Oxs_Box& other)
{
  if(other.x0>other.x1) return 1; // "other" is an empty box
  if(x0>x1) return 0; // *this is an empty box
  if(other.x0<x0 || other.x1>x1 ||
     other.y0<y0 || other.y1>y1 ||
     other.z0<z0 || other.z1>z1) return 0;
  return 1;
}
