/* FILE: vectorfieldinit.h      -*-Mode: c++-*-
 *
 * Abstract Oxs_VectorFieldInit class, derives from Oxs_Ext, for
 * initializing Oxs_MeshValue<ThreeVector> objects.
 */

#ifndef _OXS_VECTORFIELDINIT
#define _OXS_VECTORFIELDINIT

#include "oc.h"

#include "ext.h"
#include "director.h"
#include "key.h"
#include "mesh.h"
#include "meshvalue.h"
#include "threevector.h"

/* End includes */

////////////////////////////////////////////////////////////////////////
// Oxs_VectorFieldInit (generic Oxs_Ext vector field) class
class Oxs_VectorFieldInit : public Oxs_Ext {
protected:
  // Disable copy constructor and assignment operator by
  // declaring them here and never providing a definition.
  Oxs_VectorFieldInit(const Oxs_VectorFieldInit&);
  Oxs_VectorFieldInit& operator=(const Oxs_VectorFieldInit&);

  // Constructors
  Oxs_VectorFieldInit(const char* name,
		      Oxs_Director* newdtr);
  Oxs_VectorFieldInit(const char* name,Oxs_Director* newdtr,
		      Tcl_Interp* safe_interp,const char* argstr);


public:
  virtual ~Oxs_VectorFieldInit() {}

  virtual void
  FillMeshValue(const Oxs_Mesh* mesh,
		Oxs_MeshValue<ThreeVector>& array) const =0;

  // FillMeshValueOrthogonal is analogous to FillMeshValue, except
  // that the export array is filled with values orthogonal to the
  // orth_array import.  The base class provides a default implementation
  // that first calls FillMeshValue, and then checks that the orthogonality
  // relation holds at each point.  An exception should be thrown if the
  // class cannot fulfill the orthogonality requirement.
  virtual void
  FillMeshValueOrthogonal(const Oxs_Mesh* mesh,
			  const Oxs_MeshValue<ThreeVector>& orth_array,
			  Oxs_MeshValue<ThreeVector>& array) const;

};


#endif // _OXS_VECTORFIELDINIT



