/* FILE: demag.h            -*-Mode: c++-*-
 *
 * Average H demag field across rectangular cells.  This is a modified
 * version of the simpledemag class, which uses symmetries in the
 * interaction coefficients to reduce memory usage.
 *
 */

#ifndef _OXS_DEMAG
#define _OXS_DEMAG

#include "energy.h"
#include "fft.h"
#include "key.h"
#include "mesh.h"
#include "meshvalue.h"
#include "simstate.h"
#include "threevector.h"
#include "rectangularmesh.h"

/* End includes */

class Oxs_Demag:public Oxs_Energy {
private:
  mutable UINT4m xdim,ydim,zdim,totalsize;  // Natural size
  mutable UINT4m pxdim,pydim,pzdim,ptotalsize;  // Padded size
  mutable UINT4m mesh_id;
  // The A## arrays hold demag coefficients, transformed into
  // frequency domain.  These are held long term.  xcomp,
  // ycomp, and zcomp are used as temporary space, first to hold
  // the transforms of Mx, My, and Mz, then to store Hx, Hy, and
  // Hz.
  mutable REAL8m *A00,*A01,*A02,*A11,*A12,*A22;
  mutable MyComplex *xcomp,*ycomp,*zcomp;
  mutable FFT3D fft; // All transforms are same size, so we need
  /// only one FFT3D object.

  void FillCoefficientArrays(const Oxs_Mesh* mesh) const;

  UINT4m NextPowerOfTwo(UINT4m n) const;  // Helper function
  void ReleaseMemory();
public:
  virtual const char* ClassName() const; // ClassName() is
  /// automatically generated by the OXS_EXT_REGISTER macro.
  Oxs_Demag(const char* name,     // Child instance id
	    Oxs_Director* newdtr, // App director
	    Tcl_Interp* safe_interp, // Safe interpreter
	    const char* argstr);  // MIF input block parameters
  virtual ~Oxs_Demag() { ReleaseMemory(); }

  virtual void GetEnergyAndField(const Oxs_SimState& state,
                                 Oxs_MeshValue<REAL8m>& energy,
                                 Oxs_MeshValue<ThreeVector>& field
                                 ) const;
};


#endif // _OXS_DEMAG
