/* FILE: rectangularsection.cc      -*-Mode: c++-*-
 *
 * Concrete section class, derived from Oxs_Section class.
 *
 */

#include "rectangularsection.h"

// Oxs_Ext registration support
OXS_EXT_REGISTER(Oxs_RectangularSection);

/* End includes */


// Constructor
Oxs_RectangularSection::Oxs_RectangularSection(
  const char* name,     // Child instance id
  Oxs_Director* newdtr, // App director
  Tcl_Interp* safe_interp, // Safe interpreter
  const char* argstr)   // MIF input block parameters
  : Oxs_Section(name,newdtr,safe_interp,argstr)
{
  // Process arguments
  CheckInitValueParamCount("xrange",2);
  const vector<string>& xrange = *FindInitValue("xrange");

  CheckInitValueParamCount("yrange",2);
  const vector<string>& yrange = *FindInitValue("yrange");

  CheckInitValueParamCount("zrange",2);
  const vector<string>& zrange = *FindInitValue("zrange");

  box.Set(xrange[0],xrange[1],
	  yrange[0],yrange[1],
	  zrange[0],zrange[1]);

  DeleteInitValue("xrange");
  DeleteInitValue("yrange");
  DeleteInitValue("zrange");
  VerifyAllInitArgsUsed();
}


BOOL Oxs_RectangularSection::Init()
{ return 1; }

void Oxs_RectangularSection::GetExtents(Oxs_Box &mybox) const
{
  mybox=box;
}

BOOL Oxs_RectangularSection::IsIn(const ThreeVector& point) const
{ // Returns 1 if point is inside section, 0 otherwise.
  if(point.x<box.GetMinX() || point.x>=box.GetMaxX()) return 0;
  if(point.y<box.GetMinY() || point.y>=box.GetMaxY()) return 0;
  if(point.z<box.GetMinZ() || point.z>=box.GetMaxZ()) return 0;
  return 1;
}

Oxs_RectangularSection::~Oxs_RectangularSection()
{}


