/* FILE: standarddriver.h            -*-Mode: c++-*-
 *
 * Example concrete Oxs_Driver class.
 *
 */

#ifndef _OXS_STANDARDDRIVER
#define _OXS_STANDARDDRIVER

#include <memory>   // auto_ptr template

#include "simstate.h"
#include "key.h"
#include "driver.h"
#include "vectorfieldinit.h"
#include <memory>
#include <string>

/* End includes */

class Oxs_StandardDriver:public Oxs_Driver {
private:
  Oxs_ConstKey<Oxs_Mesh> meshkey;
  Oxs_MeshValue<REAL8m> Ms;  // Saturation magnetization
  Oxs_MeshValue<REAL8m> Ms_inverse;  // 1/Ms
  auto_ptr<Oxs_VectorFieldInit> m0;  // Initial spin configuration

  REAL8m min_timestep;          // Seconds
  REAL8m max_timestep;          // Seconds
  REAL8m stopping_dm_dt;        // deg/ns
  UINT4m number_of_stages;      // 0 => unlimited
  UINT4m stage_iteration_limit; // 0 => no limit
  UINT4m total_iteration_limit; // 0 => no limit

  Tcl_Interp* const safe_interp; // For obtaining output object data
  Oxs_Output* max_dm_dt_obj_ptr; // Pointer to object providing
  /// max dm/dt data.

  // State-based outputs, maintained by the driver.  These are
  // conceptually public, but are specified private to force
  // clients to use the output_map interface in Oxs_Director.
#define OSO_DECL(name) \
void Fill__##name##_output(const Oxs_SimState&); \
Oxs_ScalarOutput<Oxs_StandardDriver> name##_output
  OSO_DECL(iteration_count);
  OSO_DECL(stage_iteration_count);
  OSO_DECL(stage_number);
  OSO_DECL(last_timestep);
  OSO_DECL(simulation_time);
  ///// OSO_DECL(max_dm_dt);
  ///// OSO_DECL(total_energy);
  ///// OSO_DECL(total_energy_change);
  Oxs_VectorFieldOutput<Oxs_StandardDriver> magnetization_output;
  void Fill__magnetization_output(const Oxs_SimState&);
  Oxs_ScalarOutput<Oxs_StandardDriver> aveMx_output;
  Oxs_ScalarOutput<Oxs_StandardDriver> aveMy_output;
  Oxs_ScalarOutput<Oxs_StandardDriver> aveMz_output;
  void Fill__aveM_output(const Oxs_SimState&);

public:
  Oxs_StandardDriver(const char* name,    // Child instance id
		     Oxs_Director* newdtr, // App director
		     Tcl_Interp* safe_interp, // Safe interpreter
		     const char* argstr);  // MIF input block parameters
  virtual const char* ClassName() const; // ClassName() is
  /// automatically generated by the OXS_EXT_REGISTER macro.
  virtual BOOL Init();
  virtual ~Oxs_StandardDriver();

  // Generic interface
  virtual Oxs_ConstKey<Oxs_SimState> GetInitialState() const;
  virtual BOOL Done(const Oxs_SimState& state, string& result) const;
  virtual BOOL StageDone(const Oxs_SimState& state, string& result) const;
  virtual void NewState(const Oxs_SimState& new_state, string& result) const;

  virtual void FillStatePartA(const Oxs_SimState& old_state,
			      Oxs_SimState& new_state) const;
  virtual void FillStatePartB(Oxs_SimState& work_state) const;
};

#endif // _OXS_STANDARDDRIVER
